#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <lzma_adv.h>
#include <stdio.h>

#define BUFFER_SIZE (1U << 14)

int
main()
{
	lzma_init();

	uint8_t in[BUFFER_SIZE];
	uint8_t out[BUFFER_SIZE];

	FILE *ifile = fopen("zzz", "r");
	FILE *ofile = fopen("zzz.lzma", "w");

	lzma_stream strm = LZMA_STREAM_INIT;

// 	lzma_extra e1 = { NULL, LZMA_EXTRA_FILENAME, 6, (uint8_t *)"7z.exe" };

	{
		lzma_options_lzma lzma = {
			.dictionary_size = 1 << 17,
			.literal_context_bits = 3,
			.literal_pos_bits = 0,
			.pos_bits = 2,
			.mode = LZMA_MODE_BEST,
			.fast_bytes = 48,
			.match_finder = LZMA_MF_BT4,
			.match_finder_cycles = 0,
		};

		lzma_options_subblock subblock = {
			.allow_subfilters = 0,
			.subblock_data_size = LZMA_SUBBLOCK_DATA_SIZE_DEFAULT,
		};

		lzma_options_stream stream = {
			.check = LZMA_CHECK_SHA256,
			.has_crc32 = 1,
			.alignment = 0,
		};
		stream.filters[0].id = LZMA_FILTER_SUBBLOCK;
		stream.filters[0].options = &subblock;
// 		stream.filters[0].id = LZMA_FILTER_X86;
// 		stream.filters[0].options = NULL;
// 		stream.filters[1].id = LZMA_FILTER_COPY;
// 		stream.filters[1].options = NULL;
		stream.filters[1].id = LZMA_FILTER_LZMA;
		stream.filters[1].options = &lzma;
		stream.filters[2].id = UINT64_MAX;
// 		stream.filters[3].id = UINT64_MAX;
		stream.metadata_filters[0].id = UINT64_MAX;
		stream.metadata_filters[0].options = &lzma;
		stream.metadata_filters[1].id = UINT64_MAX;
		stream.header = NULL;
		stream.footer = NULL;
		const lzma_ret ret = lzma_stream_encoder_multi(&strm, &stream);
		if (ret != LZMA_OK) {
			fprintf(stderr, "error init: %d\n", ret);
			return 1;
		}
	}
/*
	{
		lzma_index i2 = { NULL, 0x77, 0x88 };
		lzma_index i1 = { &i2, 0x55, 0x66 };
		lzma_extra e2 = { NULL, 0, 2, (uint8_t *)"\x35\x36" };
		lzma_extra e1 = { &e2, 0, 1, (uint8_t *)"\x45" };
		lzma_metadata metadata = {
			.header_metadata_size = UINT64_MAX,
			.total_size = 0x11,
			.uncompressed_size = 0x22,
			.index = &i1,
			.extra = &e1,
		};
		lzma_metadata_encoder *coder;
		lzma_metadata_encoder_init(&coder, NULL, &metadata);
		while (1) {
			strm.next_out = out;
			strm.avail_out = BUFFER_SIZE;
			const lzma_ret ret = lzma_metadata_encode(coder, &strm);
			fwrite(out, 1, BUFFER_SIZE - strm.avail_out, ofile);
			if (ret != LZMA_OK)
				break;
		}

		lzma_metadata_encoder_end(&coder, NULL);
		return 0;
	}
*/
	while (!feof(ifile)) {
		strm.next_in = in;
		strm.avail_in = fread(in, 1, BUFFER_SIZE, ifile);

		while (strm.avail_in > 0) {
			strm.next_out = out;
			strm.avail_out = BUFFER_SIZE;

			const lzma_ret ret = lzma_code(&strm, LZMA_FLUSH);
			if (ret != LZMA_OK && ret != LZMA_STREAM_END) {
				fprintf(stderr, "error run: %d\n", ret);
				return 1;
			}

			fwrite(out, 1, BUFFER_SIZE - strm.avail_out, ofile);
		}
	}

	while (1) {
		strm.next_out = out;
		strm.avail_out = BUFFER_SIZE;

		const lzma_ret ret = lzma_code(&strm, LZMA_FINISH);
		if (ret != LZMA_OK && ret != LZMA_STREAM_END) {
			fprintf(stderr, "error finish: %d\n", ret);
			return 1;
		}

		fwrite(out, 1, BUFFER_SIZE - strm.avail_out, ofile);

		if (ret == LZMA_STREAM_END)
			break;
	}

	lzma_end(&strm);

	return 0;
}
